@@ -897,7 +897,7 @@ class Negotiation(protocol.Protocol):
             # We reject the offer to avoid connection flap, and the
             # situtation won't be worse than it was in 0.1.7 .
             lp2 = log("pre-0.2.0 peer detected (no my-incarnation"
-                      " or last-connection", level=CURIOUS)
+                      " or last-connection)", level=CURIOUS)
             if self.tub._handle_old_duplicate_connections is not False:
                 # but if we've been configured to do better (with the
                 # 60-second age heuristic), do that.
@@ -1159,7 +1159,12 @@ class Negotiation(protocol.Protocol):
                              )
         b.factory = self.factory # not used for PB code
         b.setTub(self.tub)
-        self.transport.protocol = b
+        # we leave ourselves as the protocol, but redirect incoming messages
+        # (from the transport) to the broker
+        #self.transport.protocol = b
+        self.dataReceived = b.dataReceived
+        self.connectionLost = b.connectionLost
+
         b.makeConnection(self.transport)
         buf, self.buffer = self.buffer, "" # empty our buffer, just in case
         b.dataReceived(buf) # and hand it to the new protocol
@@ -1172,6 +1177,7 @@ class Negotiation(protocol.Protocol):
 
         # finally let our Tub know that they can start using the new Broker.
         # This will wake up anyone who initiated an outbound connection.
+        self.tub.connectionAttached(b)
         self.tub.brokerAttached(theirTubRef, b, self.isClient)
 
     def negotiationFailed(self):
