@@ -311,7 +311,11 @@ class Tub(service.MultiService):
         self.unauthenticatedBrokers = [] # inbound Brokers without TubRefs
         self.reconnectors = []
 
-        self._allBrokersAreDisconnected = observer.OneShotObserverList()
+        # all connections, used or not, go in here, so we can stall
+        # stopService() until they're all gone.
+        self._allConnections = set()
+        self._allConnectionsAreDisconnected = observer.OneShotObserverList()
+
         self._activeConnectors = []
         self._allConnectorsAreFinished = observer.OneShotObserverList()
 
@@ -321,7 +325,7 @@ class Tub(service.MultiService):
         self._logport_furl = None
         self._logport_furlfile = None
 
-        self._log_gatherer_furl = None
+        self._log_gatherer_furls = []
         self._log_gatherer_furlfile = None
         self._log_gatherer_connectors = {} # maps furl to reconnector
 
@@ -376,9 +380,12 @@ class Tub(service.MultiService):
         else:
             raise KeyError("unknown option name '%s'" % name)
 
-    def setLogGathererFURL(self, gatherer_furl):
-        assert not self._log_gatherer_furl
-        self._log_gatherer_furl = gatherer_furl
+    def setLogGathererFURL(self, gatherer_furl_or_furls):
+        assert not self._log_gatherer_furls
+        if isinstance(gatherer_furl_or_furls, basestring):
+            self._log_gatherer_furls.append(gatherer_furl_or_furls)
+        else:
+            self._log_gatherer_furls.extend(gatherer_furl_or_furls)
         self._maybeConnectToGatherer()
 
     def setLogGathererFURLFile(self, gatherer_furlfile):
@@ -390,8 +397,8 @@ class Tub(service.MultiService):
         if not self.locationHints:
             return
         furls = []
-        if self._log_gatherer_furl:
-            furls.append(self._log_gatherer_furl)
+        if self._log_gatherer_furls:
+            furls.extend(self._log_gatherer_furls)
         if self._log_gatherer_furlfile:
             try:
                 # allow multiple lines
@@ -648,8 +655,8 @@ class Tub(service.MultiService):
         for c in self._activeConnectors:
             c.shutdown()
 
-        if self.brokers or self.unauthenticatedBrokers:
-            dl.append(self._allBrokersAreDisconnected.whenFired())
+        if self._allConnections:
+            dl.append(self._allConnectionsAreDisconnected.whenFired())
         why = Failure(error.ConnectionDone("Tub.stopService was called"))
         for b in self.brokers.values():
             b.shutdown(why, fireDisconnectWatchers=False)
@@ -982,6 +989,8 @@ class Tub(service.MultiService):
         b2.setTub(self)
         t1.protocol = b1; t2.protocol = b2
         b1.makeConnection(t1); b2.makeConnection(t2)
+        self.connectionAttached(b1)
+        self.connectionAttached(b2)
         self.brokerAttached(tubref, b1, False)
         return b1
 
@@ -1005,6 +1014,9 @@ class Tub(service.MultiService):
             for d in waiting:
                 d.errback(why)
 
+    def connectionAttached(self, broker):
+        self._allConnections.add(broker)
+
     def brokerAttached(self, tubref, broker, isClient):
         assert self.running
         if not tubref:
@@ -1042,24 +1054,20 @@ class Tub(service.MultiService):
         # a loopback connection will produce two Brokers that both use the
         # same tubref. Both will shut down about the same time. Make sure
         # this doesn't confuse us.
-        had_connections = (bool(self.brokers) or
-                           bool(self.unauthenticatedBrokers))
+
         # the Broker will have already severed all active references
         for tubref in self.brokers.keys():
             if self.brokers[tubref] is broker:
                 del self.brokers[tubref]
         if broker in self.unauthenticatedBrokers:
             self.unauthenticatedBrokers.remove(broker)
+
+    def connectionDropped(self, broker):
+        self._allConnections.discard(broker)
         # if the Tub has already shut down, we may need to notify observers
         # who are waiting for all of our connections to finish shutting down.
-        # Only do this if we actually transitioned from having some
-        # connections to not having any connections.
-
-        if (not self.running
-            and had_connections
-            and not self.brokers
-            and not self.unauthenticatedBrokers):
-            self._allBrokersAreDisconnected.fire(self)
+        if (not self.running and not self._allConnections):
+            self._allConnectionsAreDisconnected.fire(self)
 
     def debug_listBrokers(self):
         # return a list of (tubref, inbound, outbound) tuples. The tubref
